from enum import Enum


class EUserTaskType(Enum):
    ACTIVATE_WALLET = 1
    BUY_MERCH = 2
    ENHANCE_PROFILE = 3
    INVITE_THREE_FRIENDS = 4
    TALK_IN_CHAT_10_MINUTES = 5
    ACTIVATE_WIDGETS = 6
