from enum import Enum


class EUploadTarget(Enum):
    USER_ICON = 1
    BACKGROUND = 2
    FOREGROUND = 3
    BLOG_AUDIO = 5
    MESSAGE = 8
    MESSAGE_AUDIO = 10
    USER_NAME_CARD_BACKGROUND = 11
