from enum import Enum


class EContentRegion(Enum):
    ALL = 0
    ENGLISH = 1
    ARABIC = 2
    RUSSIAN = 3
    PORTUGUESE = 4
    SPANISH = 5
    GERMAN = 6
    FRENCH = 7
    BURMESE = 8
    OTHER = 100
